@@ -159,7 +159,7 @@ KDE_NO_CDTOR_EXPORT Playlist::Playlist (Application *a, KMPlayer::Control *s, bo
     : FileDocument (KMPlayer::id_node_playlist_document, "Playlist://", s),
       app(a),
       playmode (plmode) {
-    caption = "Persistent Playlists";
+    caption = "Favorites";
 }
 
 KDE_NO_EXPORT KMPlayer::Node *Playlist::childFromTag (const String & tag) {
@@ -196,7 +196,9 @@ KDE_NO_EXPORT void PlaylistItemBase::activate () {
         String pn;
         if (parentNode ()->id == KMPlayer::id_node_group_node &&
                 (previousSibling() || nextSibling ())) {
-            data = parentNode ()->innerXML ();
+            data = String ("<playlist>") +
+                parentNode ()->innerXML () +
+                String ("</playlist>");
             pn = parentNode ()->caption;
         } else {
             data = outerXML ();
