@@ -198,7 +198,6 @@ const short id_node_seq = 112;
 const short id_node_switch = 113;
 const short id_node_excl = 114;
 const short id_node_img = 120;
-const short id_node_audio_video = 121;
 const short id_node_text = 122;
 const short id_node_ref = 123;
 const short id_node_brush = 124;
@@ -304,7 +303,7 @@ public:
     void boundsUpdate (); // recalculates and repaint old and new bounds
 
     SurfacePtrW region_surface;
-    ImageActor *bg_image;
+    MediaInfo *media_info;
     CalculatedSizer sizes;
 
     Single x, y, w, h;     // unscaled values
@@ -529,7 +528,11 @@ public:
     void deactivate ();
     void reset ();
     void *message (MessageType msg, void *content=NULL);
-    NodePtrW chosenOne;
+    KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
+
+    Node *chosenOne ();
+private:
+    NodePtrW chosen_one;
 };
 
 class KMPLAYER_NO_EXPORT LinkingBase : public Element {
@@ -637,12 +640,11 @@ protected:
     bool inited;
 };
 
-class KMPLAYER_NO_EXPORT AVMediaType : public MediaType {
+class KMPLAYER_NO_EXPORT RefMediaType : public MediaType {
 public:
-    AVMediaType (NodePtr & d, const String & t);
+    RefMediaType (NodePtr &doc, const String &tag);
     Node *childFromTag (const String & tag);
     virtual void begin ();
-    virtual void undefer ();
     virtual void finish ();
     virtual void accept (Visitor *);
     virtual bool expose () const;
@@ -656,11 +658,9 @@ public:
     Node *childFromTag (const String & tag);
     PlayType playType () { return play_type_image; }
     virtual void activate ();
-    virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
     virtual void *message (MessageType msg, void *content=NULL);
-    void dataArrived ();
 };
 
 class KMPLAYER_NO_EXPORT TextMediaType : public MediaType {
@@ -668,11 +668,8 @@ public:
     TextMediaType (NodePtr & d);
     PlayType playType () { return play_type_info; }
     virtual void init ();
-    virtual void begin ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
-    virtual void *message (MessageType msg, void *content=NULL);
-    void dataArrived ();
 
     String font_name;
     int font_size;
@@ -682,12 +679,6 @@ public:
     enum { align_left, align_center, align_right } halign;
 };
 
-class KMPLAYER_NO_EXPORT RefMediaType : public MediaType {
-public:
-    RefMediaType (NodePtr & d);
-    virtual void accept (Visitor *);
-};
-
 class KMPLAYER_NO_EXPORT Brush : public MediaType {
 public:
     Brush (NodePtr & d);
