@@ -37,35 +37,6 @@ using namespace KMPlayer;
 #include <fcntl.h>
 #include <unistd.h>
 
-static bool isPlayListMime (const String & mime) {
-    const char * mimestr = (const char *) mime;
-    //debugLog () << "isPlayListMime  mime:" << mimestr << endl;
-    return mimestr && (!strcmp (mimestr, "audio/mpegurl") ||
-            !strcmp (mimestr, "audio/x-mpegurl") ||
-            !strncmp (mimestr, "video/x-ms", 10) ||
-            !strncmp (mimestr, "audio/x-ms", 10) ||
-            //!strcmp (mimestr, "video/x-ms-wmp") ||
-            //!strcmp (mimestr, "video/x-ms-asf") ||
-            //!strcmp (mimestr, "video/x-ms-wmv") ||
-            //!strcmp (mimestr, "video/x-ms-wvx") ||
-            //!strcmp (mimestr, "video/x-msvideo") ||
-            !strcmp (mimestr, "video/x-real") ||
-            !strcmp (mimestr, "audio/x-scpls") ||
-            !strcmp (mimestr, "audio/x-pn-realaudio") ||
-            !strcmp (mimestr, "audio/vnd.rn-realaudio") ||
-            !strcmp (mimestr, "audio/m3u") ||
-            !strcmp (mimestr, "audio/x-m3u") ||
-            !strncmp (mimestr, "text/", 5) ||
-            (!strncmp (mimestr, "application/", 12) &&
-             strstr (mimestr + 12,"+xml")) ||
-            !strncasecmp (mimestr, "application/smil", 16) ||
-            !strncasecmp (mimestr, "application/xml", 15) ||
-            !strcmp (mimestr, "application/news_reader") ||
-            !strcmp (mimestr, "application/rss+xml") ||
-            !strcmp (mimestr, "application/atom+xml") ||
-            !strcmp (mimestr, "application/x-mplayer2"));
-}
-
 #ifdef KMPLAYER_DEBUG
 static Application *debug_app;
 static Control *debug_control;
@@ -83,11 +54,6 @@ void sig_usr1 (int) {
 }
 #endif
 
-static bool isAudioOnlyMime (const String & mime) {
-    const char * mimestr = (const char *) mime;
-    return mimestr && (!strncmp ((const char *) mime, "audio/", 6));
-}
-
 static gboolean playSchedule (void * ctr) {
     static_cast <Control *> (ctr)->play ();
     return false;
@@ -104,6 +70,9 @@ SourceDocument::SourceDocument (Control *c, const String &url)
 void *SourceDocument::message (MessageType msg, void *data) {
     switch (msg) {
 
+    case MsgQueryActorAgent:
+        return m_control->actorAgent ();
+
     case MsgQueryRoleChildDisplay:
         return VIEWAREA(m_control->m_app)->getSurface ((Mrl *) data);
 
@@ -154,6 +123,12 @@ void Control::init () {
     String p = m_config->readEntry (GCONF_KEY_PLAYER, OssoMediaServer::myname);
     setProcess ((const char *) p);
 
+    if (m_config->readNumEntry (GCONF_KEY_VO, 0)) {
+        mplayer->setUseXv (false);
+        gtk_check_menu_item_set_active (
+                GTK_CHECK_MENU_ITEM (m_app->menu_item_mplayer_vo), true);
+    }
+
     GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
     addTree (m_app, 0L, //gdk_pixbuf_new_from_xpm_data((const char**)www_xpm));
             gdk_pixbuf_scale_simple (
@@ -224,6 +199,8 @@ void Control::deinit () {
     for (NodePtr n = extra_playlists.first(); n; n = n->nextSibling ())
         n->document()->dispose ();
 
+    m_config->writeEntry (GCONF_KEY_VO, mplayer->hasXv () ? 0 : 1);
+
     delete media_server;
     delete mplayer;
 #ifdef __ARMEL__
@@ -280,9 +257,6 @@ void Control::play () {
 
 void Control::stop () {
     debugLog () << "stop" << endl;
-    for (SharedPtr<ResolveInfo> rinfo =m_resolve_info; rinfo; rinfo=rinfo->next)
-        rinfo->job->kill ();
-    m_resolve_info = 0L;
     setLoading (100); // no pogress bar
     if (blanking_timer) {
         g_source_remove (blanking_timer);
@@ -420,235 +394,6 @@ static String processForMimetype (Config *config, const String & mime) {
     return pname;
 }
 
-bool Control::resolveURL (NodePtr m) {
-  //debugLog() << "resolveURL " << !!back_request << " " << m->mrl()->src<<endl;
-    if (back_request && back_request->mrl ()->view_mode == Mrl::SingleMode)
-        return false; // block auto playlist traversal on jump
-    Mrl * mrl = m->mrl ();
-    if (!mrl || mrl->src.isEmpty ())
-        return true;
-    int depth = 0;
-    for (NodePtr e = m->parentNode (); e; e = e->parentNode ())
-        ++depth;
-    if (depth > 40)
-        return true;
-    URL url (mrl->absolutePath ());
-    String mimestr = mrl->mimetype;
-    if (!mimestr.isEmpty () &&
-            processForMimetype (m_config, mimestr) == String ("npp"))
-        return true; // FIXME
-    if (mimestr.isEmpty ()) {
-        if (url.isLocalFile ())
-            mimestr = MimeType::findByURL (url);
-        if (!mimestr.isEmpty ())
-            mrl->mimetype = mimestr;
-    }
-    debugLog() << "resolveURL " << url << " mime:" << mimestr << endl;
-    if (mimestr.isEmpty ()) {
-        for (NodePtr p = mrl->parentNode (); p; p = p->parentNode ()) {
-            Mrl *m = p->mrl ();
-            if (m && m->linkNode ()->audio_only) {
-                mrl->linkNode ()->audio_only = true;
-                break;
-            }
-        }
-    } else
-        mrl->linkNode ()->audio_only |= isAudioOnlyMime (mimestr);
-    bool maybe_playlist = mimestr.isEmpty() || isPlayListMime (mimestr);
-    if (url.isLocalFile ()) {
-        File file (url.path ());
-        struct stat stats;
-        if (!file.exists ()) {
-            errorLog () << url.path () << " not found" << endl;
-            return true;
-        }
-        //debugLog () << "opening " << url << " mime:" << mimestr << endl;
-        if (maybe_playlist && file.size () < 2000000) {
-            char buf[256];
-            off_t nr = file.readBlock (buf, sizeof (buf) - 1);
-            if (nr > 0) {
-                buf[nr] = 0;
-                String mimestr = MimeType::findByContent (buf, nr);
-                if (mimestr.startsWith (String ("text/")) &&
-                        strncmp (buf, "RIFF", 4)) {
-                    ByteArray ba = file.readAll ();
-                    String data (ba.data (), ba.size ());
-                    if (!data.isEmpty ()) {
-                        TextStream in (data);
-                        readPlaylist (m, in);
-                    }
-                }
-            }
-        }
-    } else if ((maybe_playlist && // exclude mms/rtsp/rtp 
-                url.protocol ().compare (String ("mms")) &&
-                url.protocol ().compare (String ("rtsp")) &&
-                url.protocol ().compare (String ("rtp"))) ||
-            (mimestr.isEmpty () && // try http(s) ones too
-             url.protocol ().startsWith (String ("http")))) {
-        debugLog () << "maybe playlist, network " << endl;
-        Job * job = asyncGet (this, url.url ());
-        m_resolve_info = new ResolveInfo (m, job, m_resolve_info);
-        return false; // wait for network operation ..
-    }
-    return true;
-}
-
-void Control::jobData (Job * job, ByteArray & data) {
-    SharedPtr <ResolveInfo> rinfo = m_resolve_info;
-    while (rinfo && rinfo->job != job)
-        rinfo = rinfo->next;
-    if (!rinfo) {
-        warningLog () << "Spurious kioData" << endl;
-        return;
-    }
-    //debugLog () << "jobData of " << data.size () << endl;
-    if (rinfo->data.isEmpty ()) {
-        String mimestr = MimeType::findByContent (data.data (), data.size ());
-        if (rinfo->resolving_mrl->mrl ()->mimetype.isEmpty ())
-            rinfo->resolving_mrl->mrl ()->mimetype =
-                job->contentType ().isEmpty () ? mimestr : job->contentType ();
-        if (mimestr.isEmpty() || !mimestr.startsWith (String ("text/")) ||
-                (data.size () > 4 && !strncmp (data.data (), "RIFF", 4)) ||
-                !rinfo->resolving_mrl) {
-            job->kill (false); // not quiet, wants jobResult
-            return;
-        }
-    }
-    rinfo->data.readRawBytes (data.data (), data.size ());
-}
-
-void Control::jobResult (Job * job) {
-    debugLog () << "jobResult" << endl;
-    SharedPtr <ResolveInfo> previnfo, rinfo = m_resolve_info;
-    while (rinfo && rinfo->job != job) {
-        previnfo = rinfo;
-        rinfo = rinfo->next;
-    }
-    if (!rinfo) {
-        warningLog () << "Spurious kioData" << endl;
-        return;
-    }
-    if (previnfo)
-        previnfo->next = rinfo->next;
-    else
-        m_resolve_info = rinfo->next;
-    if (rinfo->resolving_mrl) {
-        Mrl *mrl = rinfo->resolving_mrl->mrl ();
-        if (!rinfo->data.isEmpty ())
-            readPlaylist (mrl, rinfo->data);
-    //debugLog () << "set resolved " << rinfo->resolving_mrl->mrl ()->src << endl;
-        mrl->resolved = true;
-        if (mrl->mimetype.isEmpty ())
-            mrl->mimetype = job->contentType ();
-        mrl->audio_only = isAudioOnlyMime (mrl->mimetype);
-        mrl->undefer ();
-    }
-}
-
-void Control::readPlaylist (NodePtr root, TextStream & in) {
-    String line;
-    Mrl * mrl = root->mrl ();
-    do {
-        line = in.readLine ().stripWhiteSpace ();
-    } while (!in.atEnd () && line.isEmpty ());
-    if (!mrl || line.isEmpty ())
-        return;
-    NodePtr cur_elm = root;
-    if (mrl->isPlayable ())
-        cur_elm = mrl->linkNode ();
-    if (mrl->mimetype.startsWith ("text/") && line.indexOf ("[playlist]") > -1)
-        mrl->mimetype = "audio/x-scpls";
-    if (mrl->mimetype == String ("audio/x-scpls")) {
-        bool groupfound = false;
-        int nr = -1;
-        struct Entry {
-            String url, title;
-        } * entries = 0L;
-        do {
-            if (line.startsWith (String ("[")) && line.endsWith(String ("]"))) {
-                if (!groupfound && line.mid (1, line.length () - 2).stripWhiteSpace () == String ("playlist"))
-                    groupfound = true;
-                else
-                    break;
-                //debugLog () << "Group found: " << line << endl;
-            } else if (groupfound) {
-                int eq_pos = line.indexOf (Char ('='));
-                if (eq_pos > 0) {
-                    if (line.lower ().startsWith (String ("numberofentries"))) {
-                        nr = line.mid (eq_pos + 1).stripWhiteSpace ().toInt ();
-                        debugLog () << "numberofentries : "<< nr<< endl;
-                        if (nr > 0 && nr < 1024)
-                            entries = new Entry[nr];
-                        else
-                            nr = 0;
-                    } else if (nr > 0) {
-                        String ll = line.lower ();
-                        if (ll.startsWith (String ("file"))) {
-                            int i = line.mid (4, eq_pos-4).toInt ();
-                            if (i > 0 && i <= nr)
-                                entries[i-1].url = line.mid (eq_pos + 1).stripWhiteSpace ();
-                        } else if (ll.startsWith (String ("title"))) {
-                            int i = line.mid (5, eq_pos-5).toInt ();
-                            if (i > 0 && i <= nr)
-                                entries[i-1].title = line.mid (eq_pos + 1).stripWhiteSpace ();
-                        }
-                    }
-                }
-            }
-            line.truncate (0);
-            while (!in.atEnd () && line.isEmpty ())
-                line = in.readLine ().stripWhiteSpace ();
-        } while (!line.isEmpty ());
-        for (int i = 0; i < nr; i++)
-            if (!entries[i].url.isEmpty ()) {
-                Mrl * m = new GenericURL (playlist_data->document, URL::decode_string (entries[i].url), entries[i].title);
-                cur_elm->appendChild (m);
-                m->linkNode ()->audio_only = true;
-            }
-        delete [] entries;
-    } else if (line.stripWhiteSpace ().startsWith (Char ('<'))) {
-        readXML (cur_elm, in, line);
-        //cur_elm->normalize ();
-        if (playlist_data->document && playlist_data->document->firstChild ()) {
-            // SMIL documents have set its size of root-layout
-            Mrl * mrl = playlist_data->document->firstChild ()->mrl ();
-            if (mrl) {
-                width = mrl->width;
-                height = mrl->height;
-                aspect = height > 0 ? width/height : 0.0;
-            }
-        }
-    } else if (line.lower () != String ("[reference]")) {
-        bool extm3u = line.startsWith ("#EXTM3U");
-        String title;
-        if (extm3u)
-            line = in.readLine ().stripWhiteSpace ();
-        while (!line.isNull ()) {
-            if (!line.isEmpty ()) {
-                if (line == String ("--stop--"))
-                    break;
-                if (line.lower ().startsWith (String ("asf ")))
-                    line = line.mid (4).stripWhiteSpace ();
-                if (!line.isEmpty ()) {
-                    if (extm3u && line.startsWith ("#")) {
-                        if (line.startsWith ("#EXTINF:"))
-                            title = line.mid (9);
-                        else
-                            title = line.mid (1);
-                    } else if (!line.startsWith (Char ('#'))) {
-                        cur_elm->appendChild (
-                                new GenericURL (playlist_data->document,
-                                    line, title));
-                        title.truncate (0);
-                    }
-                }
-                line = in.readLine ().stripWhiteSpace ();
-            }
-        }
-    }
-}
-
 ActorAgent *Control::actorAgent () const {
     return actor_agent;
 }
@@ -771,6 +516,7 @@ bool Control::setCurrent (NodePtr n) {
     if (playlist_data->current == n)
         return true;
     playlist_data->current = n;
+    current_selected = n;
     updatePlaylistView ();
     Mrl * mrl = n ? n->mrl () : 0L;
     if (mrl && (playlist_data->current->id == SMIL::id_node_smil ||
@@ -929,9 +675,11 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
         } else
             g_timeout_add (0, playCurrentSchedule, static_cast <void *> (this));
     } else if (ns == Process::Buffering) {
-        if (p->mrl ()->mrl ()->view_mode != Mrl::SingleMode) {
-            AudioVideoActor *actor =
-                static_cast <AudioVideoActor *> (p->mrl()->mrl()->actor);
+        Mrl *mrl = p->mrl ();
+        if (mrl->view_mode != Mrl::SingleMode) {
+            AudioVideoActor *actor = mrl->media_info
+                ? static_cast <AudioVideoActor *> (mrl->media_info->media)
+                : NULL;
             if (actor)
                 actor->ignore_pause = true;
             p->mrl ()->defer ();
@@ -948,10 +696,12 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
         else
             playlist_data->document->defer ();
     } else if (ns == Process::Playing) {
-        p->mrl ()->audio_only |= !p->hasVideo ();
-        if (p->mrl ()->state == Element::state_deferred) {
-            AudioVideoActor *actor =
-                static_cast <AudioVideoActor *> (p->mrl()->actor);
+        Mrl *mrl = p->mrl ()->mrl ();
+        mrl->audio_only |= !p->hasVideo ();
+        if (mrl->state == Element::state_deferred) {
+            AudioVideoActor *actor = mrl->media_info
+                ? static_cast <AudioVideoActor *> (mrl->media_info->media)
+                : NULL;
             if (actor)
                 actor->ignore_pause = true;
             p->mrl ()->undefer ();
@@ -1190,6 +940,48 @@ void cb_playlist_stop (GtkButton * button, void *p) {
     }
 }
 
+void cb_favorites_add (Application *app) {
+    Control *ctr = CONTROL(app);
+    bool valid = false;
+    gchar *msg = "Unknown failure";
+    if (!ctr->current_selected) {
+        msg = "No current selected";
+    } else {
+        Mrl *mrl = ctr->current_selected->mrl ();
+        if (!mrl || mrl->src.isEmpty ()) {
+            msg = "Not a valid link selected";
+        } else {
+            PlayListData *pd, *fav = NULL;
+            for (pd = ctr->playlist_data; pd; pd = pd->next) {
+                if (pd->id == app->playlists_tree_id)
+                    fav = pd;
+                if (pd->document.ptr () == mrl->document ()) {
+                    if (fav == pd)
+                        msg = "Already in favorites";
+                    else
+                        valid = true;
+                }
+            }
+            if (fav && valid) {
+                msg = "Added to favorites";
+                NodePtr doc = mrl->document ();
+                NodePtr n;
+                if (id_node_playlist_item == mrl->id) {
+                    n = mrl;
+                } else {
+                    n = new PlaylistItem (doc, app, false, mrl->src);
+                    n->caption = mrl->caption;
+                }
+                String xml = n->outerXML ();
+                TextStream inxml ((const char *) xml);
+                readXML (fav->document, inxml, String(), false);
+                updateTree (app, fav->id);
+            }
+        }
+    }
+    hildon_banner_show_information (GTK_WIDGET (app->window), NULL, msg);
+}
+
 void openFile (Application * app, const gchar * path) {
     CONTROL(app)->stop ();
     PlayListData *pd = CONTROL(app)->playlist_data;
@@ -1326,6 +1118,10 @@ void toggleMediaServer (GtkCheckMenuItem *radio, Application *app) {
         CONTROL(app)->m_config->writeEntry(GCONF_KEY_PLAYER, String(p->name()));
 }
 
+void toggleVO (GtkCheckMenuItem *chk, Application *app) {
+        CONTROL(app)->mplayer->setUseXv (!gtk_check_menu_item_get_active (chk));
+}
+
 void app_set_suspend (Application *a, bool suspend) {
     PlayListData *pd = CONTROL(a)->playlist_data;
     if (pd->document &&
