@@ -307,7 +307,6 @@ public:
     void visit (SMIL::Brush *);
     void visit (SMIL::SmilText *);
     void visit (SMIL::RefMediaType *);
-    void visit (SMIL::AVMediaType *);
     void visit (RP::Imfl *);
     void visit (RP::Fill *);
     void visit (RP::Fadein *);
@@ -462,9 +461,10 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
         IRect clip_save = clip;
         clip = clip.intersect (IRect (x, y, w, h));
         cairo_save (cr);
-        ImageData *bg_img = reg->bg_image && !reg->bg_image->isEmpty()
-            ? reg->bg_image->cached_img.ptr ()
+        ImageActor *im = reg->media_info
+            ? (ImageActor *) reg->media_info->media
             : NULL;
+        ImageData *bg_img = im && !im->isEmpty() ? im->cached_img.ptr () : NULL;
         if ((SMIL::RegionBase::ShowAlways == reg->show_background ||
                     reg->m_AttachedMediaTypes->first ()) &&
                 (s->background_color & 0xff000000 || bg_img)) {
@@ -659,9 +659,10 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
-    if (m->actor &&
-            ActorAgent::AudioVideo == m->actor->type ()) {
-        AudioVideoActor *avm = static_cast<AudioVideoActor *> (m->actor);
+    if (m->media_info &&
+            m->media_info->media &&
+            ActorAgent::AudioVideo == m->media_info->type) {
+        AudioVideoActor *avm = static_cast<AudioVideoActor *> (m->media_info->media);
         if (s && avm && strcmp (m->nodeName (), "audio")) {
             s->xscale = s->yscale = 1; // either scale width/heigt or use bounds
             view_widget->setAudioVideoGeometry (s->toScreen (
@@ -721,8 +722,8 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
             (mrl->id >= RP::id_node_first &&
              mrl->id < RP::id_node_last);
         if (*rp_or_smil ||
-                (mrl->actor &&
-                 ActorAgent::AudioVideo == mrl->actor->type ()))
+                (mrl->media_info &&
+                 ActorAgent::AudioVideo == mrl->media_info->type))
             return mrl;
     }
     for (Node *c = n->firstChild ().ptr (); c; c = c->nextSibling ())
@@ -770,16 +771,10 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
     paint (av, s.ptr (), x, y, clip_rect);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::AVMediaType *av) {
-    Surface *s = (Surface *) av->message (MsgQueryRoleDisplay);
-    if (s && av->external_tree)
-        updateExternal (av, s);
-    else
-        video (av, s);
-}
-
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     //debugLog() << "Visit " << img->nodeName() << " " << img->src << endl;
+    if (!img->media_info)
+        return;
     Surface *s = (Surface *) img->message (MsgQueryRoleDisplay);
     if (!s)
         return;
@@ -787,7 +782,9 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
         updateExternal (img, s);
         return;
     }
-    ImageActor *im = static_cast <ImageActor *> (img->actor);
+    if (!img->media_info->media)
+        return;
+    ImageActor *im = static_cast <ImageActor *> (img->media_info->media);
     ImageData *id = im ? im->cached_img.ptr () : NULL;
     if (!id || im->isEmpty () || img->width <= 0 || img->height <= 0) {
         s->remove();
@@ -813,8 +810,10 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
-    TextActor *tm = static_cast <TextActor *> (txt->actor);
-    Surface *s = tm ? (Surface *) txt->message (MsgQueryRoleDisplay) : NULL;
+    if (!txt->media_info || !txt->media_info->media)
+        return;
+    TextActor *tm = static_cast <TextActor *> (txt->media_info->media);
+    Surface *s = (Surface *) txt->message (MsgQueryRoleDisplay);
     //debugLog() << "Visit " << txt->nodeName() << " " << td->text << endl;
     if (!s)
         return;
@@ -1052,7 +1051,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
     //debugLog() << "Visit " << fi->nodeName() << endl;
     if (fi->target && fi->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (fi->target);
-        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
+        ImageActor *im = img && img->media_info
+            ? static_cast <ImageActor*> (img->media_info->media) : NULL;
         if (im && img->surface ()) {
             Single sx = fi->srcx, sy = fi->srcy, sw = fi->srcw, sh = fi->srch;
             if (!(int)sw)
@@ -1104,7 +1104,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
     //debugLog() << "Visit " << cf->nodeName() << endl;
     if (cf->target && cf->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (cf->target);
-        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
+        ImageActor *im = img && img->media_info
+            ? static_cast <ImageActor*> (img->media_info->media) : NULL;
         if (im && img->surface ()) {
             Single sx = cf->srcx, sy = cf->srcy, sw = cf->srcw, sh = cf->srch;
             if (!(int)sw)
@@ -1142,7 +1143,8 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
     //debugLog() << "Visit " << wipe->nodeName() << endl;
     if (wipe->target && wipe->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (wipe->target);
-        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
+        ImageActor *im = img && img->media_info
+            ? static_cast <ImageActor*> (img->media_info->media) : NULL;
         if (im && img->surface ()) {
             Single x = wipe->x, y = wipe->y;
             Single tx = x, ty = y;
@@ -1949,6 +1951,9 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
     if (event->xany.window != GDK_DRAWABLE_XID (view_area->controls))
         return GDK_FILTER_CONTINUE;
 
+    static int downx, downy;
+    static Time down_time;
+
     switch (event->xany.type) {
     case ButtonPress:
         if (app->fullscreen) {
@@ -1957,23 +1962,32 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
                 previousSelected (app);
                 x = 0;
                 w = 100;
-            } else if (event->xbutton.x > 540 && event->xbutton.x < 610) {
+            } else if (event->xbutton.x > 705) {
                 nextSelected (app);
-                x = 540;
-                w = 70;
-            } else if (event->xbutton.x > 610 && event->xbutton.x < 720) {
-                rowSelected (app);
-                x = 610;
-                w = 110;
-            } else if (event->xbutton.x > 720) {
-                view_area->controlsVisible (false);
-                x = 720;
-                w = 80;
+                x = 705;
+                w = 95;
+            } else if (event->xbutton.x > 655 && event->xbutton.x < 705) {
+                cb_favorites_add (app);
+                x = 655;
+                w = 50;
+            } else if (event->xbutton.x > 110  && event->xbutton.x < 655) {
+                downx = event->xbutton.x;
+                down_time = event->xbutton.time;
             }
             if (w > 0) {
                 gdk_window_clear_area_e (VIEWAREA(app)->controls, x, 0, w, 60);
-                if (x < 720)
-                    view_area->controlsVisible (true);
+                view_area->controlsVisible (true);
+            }
+        }
+        break;
+    case ButtonRelease:
+        if (event->xbutton.x > 110  && event->xbutton.x < 655) {
+            int dx = event->xbutton.x - downx;
+            if (-20 < dx && dx < 20 && event->xbutton.time - down_time < 1000) {
+                gdk_window_clear_area_e (VIEWAREA(app)->controls, 110, 0, 545, 60);
+                rowSelected (app);
+            } else {
+                view_area->controlsVisible (false);
             }
         }
         break;
@@ -1983,13 +1997,12 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
             cairo_t *cr = gdk_cairo_create (view_area->controls);
             cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
             cairo_set_source_rgb (cr, 32.0 / 255, 32.0 / 255, 1);
-            cairo_rectangle (cr, 0, 0, 610, 60);
+            cairo_rectangle (cr, 0, 0, 655, 60);
+            cairo_fill (cr);
+            cairo_rectangle (cr, 705, 0, 95, 60);
             cairo_fill (cr);
             cairo_set_source_rgb (cr, 32.0 / 255, 1, 48.0 / 255);
-            cairo_rectangle (cr, 610, 0, 105, 60);
-            cairo_fill (cr);
-            cairo_set_source_rgb (cr, 1, 32.0 / 255, 48.0 / 255);
-            cairo_rectangle (cr, 720, 0, 80, 60);
+            cairo_rectangle (cr, 655, 0, 50, 60);
             cairo_fill (cr);
             GValue value = { 0 , };
             if (playlist_selected (app, &value)) {
@@ -1997,7 +2010,7 @@ GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
                 cairo_set_source_rgb (cr, 0.8, 0.8, 0);
                 cairo_move_to (cr, 130, 10);
                 PangoLayout *layout = pango_cairo_create_layout (cr);
-                pango_layout_set_width (layout, 380 * PANGO_SCALE);
+                pango_layout_set_width (layout, 510 * PANGO_SCALE);
                 pango_layout_set_text (layout, val, -1);
                 PangoFontDescription *desc = pango_font_description_new ();
                 pango_font_description_set_family (desc, "Sans");
@@ -2100,7 +2113,62 @@ void cb_view_area_realized (GtkWidget *widget, Application * app) {
     }
 }
 
-#include "controls.xbm"
+static GdkBitmap *createControlsMask () {
+    cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_A1, 800, 60);
+    cairo_t *cr = cairo_create (surface);
+    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+    cairo_set_source_rgb (cr, 1, 1, 1);
+
+    cairo_new_path (cr);
+    cairo_move_to (cr, 30, 30);
+    cairo_line_to (cr, 90, 0);
+    cairo_curve_to (cr, 90 - 12, 20, 90 - 12, 40, 90, 60);
+    cairo_line_to (cr, 30, 30);
+    cairo_close_path (cr);
+    cairo_fill (cr);
+
+    cairo_new_path (cr);
+    cairo_move_to (cr, 130, 0);
+    cairo_line_to (cr, 650, 0);
+    cairo_curve_to (cr, 650 - 12, 20, 650 - 12, 40, 650, 60);
+    cairo_line_to (cr, 130, 60);
+    cairo_curve_to (cr, 130 - 12, 40, 130 - 12, 20, 130, 0);
+    cairo_close_path (cr);
+    cairo_fill (cr);
+
+    cairo_new_path (cr);
+    cairo_move_to (cr, 710, 0);
+    cairo_line_to (cr, 770, 30);
+    cairo_line_to (cr, 710, 60);
+    cairo_curve_to (cr, 710 + 12, 40, 710 + 12, 20, 710, 0);
+    cairo_close_path (cr);
+    cairo_fill (cr);
+
+    cairo_translate (cr, 680, 12);
+    cairo_new_path (cr);
+    for (int i = 0; i < 6; ++i) {
+        cairo_rotate (cr, 60 * M_PI / 180);
+        cairo_line_to (cr, 0, -8);
+        cairo_translate (cr, 0, -8);
+        cairo_rotate (cr, 60 * M_PI / 180);
+        cairo_line_to (cr, 0, -8);
+        cairo_translate (cr, 0, -8);
+        cairo_rotate (cr, 60 * M_PI / 180);
+        cairo_line_to (cr, 0, -8);
+        cairo_translate (cr, 0, -8);
+        cairo_rotate (cr, 240 * M_PI / 180);
+    }
+    cairo_close_path (cr);
+    cairo_fill (cr);
+
+    GdkBitmap *bitmap = gdk_bitmap_create_from_data (NULL,
+            (const gchar *) cairo_image_surface_get_data (surface), 800, 60);
+
+    cairo_destroy (cr);
+    cairo_surface_destroy (surface);
+
+    return bitmap;
+}
 
 gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a) {
     ViewArea *view_area = VIEWAREA(a);
@@ -2117,14 +2185,14 @@ gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a) {
         attributes.visual = gtk_widget_get_visual (w);
         attributes.colormap = gtk_widget_get_colormap (w);
         attributes.event_mask = gtk_widget_get_events (w) |
-            GDK_SUBSTRUCTURE_MASK | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK;
+            GDK_SUBSTRUCTURE_MASK | GDK_EXPOSURE_MASK |
+            GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;
         attributes.event_mask &= ~GDK_KEY_PRESS_MASK;
         //    ~(GDK_KEY_PRESS_MASK | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
         gint mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
         view_area->controls = gdk_window_new (a->view_area_fullscreen->window, &attributes, mask);
         gdk_window_add_filter (view_area->controls, (GdkFilterFunc) controlsFilter, a);
-        GdkBitmap *bitmap = gdk_bitmap_create_from_data (NULL,
-                (const gchar *) controls_bits, controls_width, controls_height);
+        GdkBitmap *bitmap = createControlsMask ();
         gdk_window_shape_combine_mask (view_area->controls, bitmap, 0, 0);
         gdk_window_show (view_area->controls);
     } else {
